@@ -49,6 +49,8 @@
 -include_lib("kernel/include/inet.hrl").
 -include_lib("kernel/include/inet_sctp.hrl").
 
+-include("s1gw_metrics.hrl").
+
 %% ------------------------------------------------------------------
 %% public API
 %% ------------------------------------------------------------------
@@ -104,6 +106,8 @@ connecting(state_timeout, conn_est_timeout, _S) ->
 %% Handle an eNB -> MME data forwarding request (queue)
 connecting(cast, {send_data, Data},
            #{tx_queue := Pending} = S) ->
+    s1gw_metrics:ctr_inc(?S1GW_CTR_S1AP_PROXY_UPLINK_PACKETS_QUEUED),
+    s1gw_metrics:gauge_inc(?S1GW_GAUGE_S1AP_PROXY_UPLINK_PACKETS_QUEUED),
     {keep_state, S#{tx_queue := [Data | Pending]}};
 
 %% Handle an #sctp_assoc_change event (connection state)
@@ -161,11 +165,19 @@ connected(info, {sctp, _Socket, MmeAddr, MmePort,
 %% Handle an #sctp_sndrcvinfo event (MME -> eNB data)
 connected(info, {sctp, _Socket, MmeAddr, MmePort,
                  {[#sctp_sndrcvinfo{assoc_id = Aid}], Data}},
-          #{enb_aid := EnbAid, priv := Priv} = S) ->
+          #{sock := Sock,
+            enb_aid := EnbAid,
+            mme_aid := Aid,
+            priv := Priv} = S) ->
     ?LOG_DEBUG("MME connection (id=~p, ~p:~p) -> eNB: ~p",
                [Aid, MmeAddr, MmePort, Data]),
-    {NewData, NewPriv} = handle_pdu(Data, Priv),
-    sctp_server:send_data(EnbAid, NewData),
+    {Action, NewPriv} = s1ap_proxy:process_pdu_safe(Data, Priv),
+    case Action of
+        {forward, FwdData} ->
+            sctp_server:send_data(EnbAid, FwdData);
+        {reply, ReData} ->
+            ok = sctp_client:send_data({Sock, Aid}, ReData)
+    end,
     {keep_state, S#{priv := NewPriv}};
 
 %% Handle termination events of the child processes
@@ -212,25 +224,19 @@ terminate(Reason, State, S) ->
 %% private API
 %% ------------------------------------------------------------------
 
-%% A safe wrapper for s1ap_proxy:handle_pdu/1
--spec handle_pdu(binary(), term()) -> {binary(), term()}.
-handle_pdu(Data, Priv) when is_binary(Data) ->
-    try s1ap_proxy:handle_pdu(Data, Priv) of
-        Result -> Result %% {NewData, NewPriv}
-    catch
-        Exception:Reason:StackTrace ->
-            ?LOG_ERROR("An exception occurred: ~p, ~p, ~p", [Exception, Reason, StackTrace]),
-            {Data, Priv} %% proxy as-is
-    end.
-
-
 %% Send a single message to the MME
 sctp_send(Data,
           #{sock := Sock,
+            enb_aid := EnbAid,
             mme_aid := Aid,
             priv := Priv} = S) ->
-    {NewData, NewPriv} = handle_pdu(Data, Priv),
-    ok = sctp_client:send_data({Sock, Aid}, NewData),
+    {Action, NewPriv} = s1ap_proxy:process_pdu_safe(Data, Priv),
+    case Action of
+        {forward, FwdData} ->
+            ok = sctp_client:send_data({Sock, Aid}, FwdData);
+        {reply, ReData} ->
+            sctp_server:send_data(EnbAid, ReData)
+    end,
     S#{priv := NewPriv}.
 
 
@@ -240,6 +246,7 @@ sctp_send_pending(#{tx_queue := Pending} = S) ->
 
 sctp_send_pending([Data | Pending], S0) ->
     S1 = sctp_send(Data, S0),
+    s1gw_metrics:gauge_dec(?S1GW_GAUGE_S1AP_PROXY_UPLINK_PACKETS_QUEUED),
     sctp_send_pending(Pending, S1);
 
 sctp_send_pending([], S) ->
